#! /usr/bin/env python

import pysam

def getSeq(reference, region_tuple):
    region_tuple_sorted = sorted(region_tuple, key=lambda x: x[1])

    seq = ""    
    for reg in region_tuple_sorted:
        for item in pysam.faidx(reference, reg[0] + ":" + str(reg[1]) + "-" + str(reg[2])):
            if item[0] == ">": continue
            seq = seq + item.rstrip('\n')

    return(seq)

def reverseComplement(seq):
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A'}
    return("".join(complement.get(base, base) for base in reversed(seq)))





#!/usr/bin/env python

import sys, os, subprocess, pysam

def sortBedpe(inputFile, outputFile):

    hOUT = open(outputFile, "w")
    subprocess.call(["sort", "-k1,1", "-k2,2n", "-k4,4", "-k5,5n", inputFile], stdout = hOUT)
    hOUT.close()

def make_directory(inputDir):
    """
    make input directory if it does not exist.
    """
    if not os.path.exists(inputDir):
        os.makedirs(inputDir)




#! /usr/bin/env python

# quit using ConfigParser... Y.S. 8/19, 2016
# import ConfigParser
 
# global param_conf

# param_conf = ConfigParser.SafeConfigParser()

global param_conf

class Param_conf(object):

    def __init__(self, reference_genome = None, resource_dir = None, debug = None,
                       abnormal_insert_size = None, min_major_clipping_size = None,
                       min_read_pair_num = None, min_valid_read_pair_ratio = None, min_cover_size = None,
                       anchor_size_thres = None, min_chimeric_size = None, min_allowed_contig_match_diff = None, 
                       check_contig_size_other_breakpoint = None, filter_same_gene = None):

        self.reference_genome = reference_genome
        self.resource_dir = resource_dir
        self.debug = debug
        self.abnormal_insert_size = abnormal_insert_size
        self.min_major_clipping_size = min_major_clipping_size
        self.min_read_pair_num = min_read_pair_num
        self.min_valid_read_pair_ratio = min_valid_read_pair_ratio
        self.min_cover_size = min_cover_size
        self.anchor_size_thres = anchor_size_thres
        self.min_chimeric_size = min_chimeric_size
        self.min_allowed_contig_match_diff = min_allowed_contig_match_diff
        self.check_contig_size_other_breakpoint = check_contig_size_other_breakpoint
        self.filter_same_gene = filter_same_gene

param_conf = Param_conf()



#! /usr/bin/env python

from run import *
import argparse

def create_parser():

    parser = argparse.ArgumentParser(prog = "fusionfusion")

    parser.add_argument("--version", action = "version", version = "fusionfusion-0.4.0")

    parser.add_argument("--star", metavar = "star.Chimeric.out.sam", default = None, type = str,
                        help = "the path to the chimeric sam file by STAR")

    parser.add_argument("--ms2", metavar = "ms2.bam", default = None, type = str,
                        help = "the path to the bam file by Map splice2")

    parser.add_argument("--th2", metavar = "th2.bam", default = None, type = str,
                        help = "the path to the bam file by TopHat2")

    parser.add_argument("--out", metavar = "output_dir", default = None, type = str, required=True,
                        help = "the path to the output directory")

    parser.add_argument("--reference_genome", metavar = "reference.fa", default = None, type = str, required=True,
                        help = "reference genome used for creating validation sequences")

    parser.add_argument("--grc", default = False, action = 'store_true',
                        help = "convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    parser.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                        help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    # parser.add_argument("--resource_dir", metavar = "resource_dir", type = str, required=True,
    #                     help = "annotation information directory")

    parser.add_argument("--pooled_control_file", default = None, type = str,
                        help = "the path to control data created by merge_control (default: %(default)s)")

    parser.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")


    parse_junction_group = parser.add_argument_group("parse_junction_condition",
                                                           "parameters used for parsing breakpoint containing read pairs from bam files")

    parse_junction_group.add_argument("--abnormal_insert_size", type = int, default = 500000,
                                      help = "size of abnormal insert size. used for checking the consistency of paired read of breakpoint containing reads (default: %(default)s)")

    parse_junction_group.add_argument("--min_major_clipping_size", type = int, default = 15,
                                      help = "minimum number of clipped bases for junction read (default: %(default)s)")


    filter_condition_group = parser.add_argument_group("filter_condition",
                                                            "parameters used in various filtering steps")

    filter_condition_group.add_argument("--min_read_pair_num", type = int, default = 3,
                                       help = "minimum required number of supporting junction read pairs (default: %(default)s)")

    filter_condition_group.add_argument("--min_valid_read_pair_ratio", type = float, default = 0.8,
                                       help = "minimum ratio of proper supporting junction read pairs (used for map-splice2) (default: %(default)s)")

    filter_condition_group.add_argument("--min_cover_size", type = int, default = 30,
                                        help = "region size which have to be covered by aligned short reads (default: %(default)s)")

    filter_condition_group.add_argument("--anchor_size_thres", type = int, default = 10,
                                       help = "at least an anchor size of one chimeric read have to be equal or larger than the specified value (default: %(default)s)")
     
    filter_condition_group.add_argument("--min_chimeric_size", type = int, default = 1000,
                                        help = "threshold of minimum chimeric transcript sizes (default: %(default)s)")

    filter_condition_group.add_argument("--min_allowed_contig_match_diff", type = int, default = 3,
                                        help = "if contigs are aligned on other positions with less than the specified mismatch value, \
                                                then the corresponding fusion transcipts are filtered (default: %(default)s)")
     
    filter_condition_group.add_argument("--check_contig_size_other_breakpoint", type = int, default = 300,
                                        help = "if contigs are aligned on within the specified size from the other breakpoint, \
                                                then the corresponding fusion transcripts are filtered (default: %(default)s)")

    filter_condition_group.add_argument("--filter_same_gene", default = False, action = 'store_true', 
                                        help = "if two breakpoints are on the same gene, then the fusion candidate is filtered %(default)s)")


    return parser




#! /usr/bin/env python

import sys, os, argparse, subprocess, shutil
import parseJunctionInfo
import filterJunctionInfo
import annotationFunction
import utils

# import config
from config import *

def cluster_filter_junction(inputFilePath, outputFilePrefix, args):

    # debug_mode = config.param_conf.getboolean("debug", "debug_mode")
    debug_mode = param_conf.debug

    parseJunctionInfo.clusterJuncInfo(inputFilePath,
                                      outputFilePrefix + ".chimeric.clustered.txt")

    filterJunctionInfo.filterCoverRegion(outputFilePrefix + ".chimeric.clustered.txt",
                                         outputFilePrefix + ".chimeric.clustered.filt1.txt")

    if args.pooled_control_file is not None:
        filterJunctionInfo.filterPoolControl(outputFilePrefix + ".chimeric.clustered.filt1.txt",
                                             outputFilePrefix + ".chimeric.clustered.filt2.txt",
                                             args.pooled_control_file)
    else:
        shutil.copyfile(outputFilePrefix + ".chimeric.clustered.filt1.txt",
                         outputFilePrefix + ".chimeric.clustered.filt2.txt")

    filterJunctionInfo.extractSplicingPattern(outputFilePrefix + ".chimeric.clustered.filt2.txt", 
                                              outputFilePrefix + ".chimeric.clustered.splicing.txt")

    ############
    filterJunctionInfo.makeJucSeqPairFa(outputFilePrefix + ".chimeric.clustered.splicing.txt",
                                        outputFilePrefix + ".chimeric.clustered.splicing.contig.fa")

    # alignment of contigs generated by manual assembly
    # blat_path = config.param_conf.get("alignment", "blat_path")
    # blat_options = config.param_conf.get("alignment", "blat_option").split(" ")
    # reference_genome = config.param_conf.get("alignment", "reference_genome")
    blat_path = "blat"
    blat_options = ["-stepSize=5", "-repMatch=2253"]
    reference_genome = param_conf.reference_genome

    FNULL = open(os.devnull, 'w')
    fRet = subprocess.check_call([blat_path] + blat_options + [reference_genome, 
                            outputFilePrefix + ".chimeric.clustered.splicing.contig.fa",
                            outputFilePrefix + ".chimeric.clustered.splicing.contig.psl"], stdout = FNULL, stderr = subprocess.STDOUT)

    FNULL.close()
    if fRet != 0:
        print >> sys.stderr, "blat error, error code: " + str(fRet)
        sys.exit()

    filterJunctionInfo.checkMatching(outputFilePrefix + ".chimeric.clustered.splicing.contig.psl",
                                     outputFilePrefix + ".chimeric.clustered.splicing.contig.check.txt")

    filterJunctionInfo.filterContigCheck(outputFilePrefix + ".chimeric.clustered.splicing.txt",
                                         outputFilePrefix + ".chimeric.clustered.filt3.txt",
                                         outputFilePrefix + ".chimeric.clustered.splicing.contig.check.txt")
    
    annotationFunction.filterAndAnnotation(outputFilePrefix + ".chimeric.clustered.filt3.txt",
                                           outputFilePrefix + ".fusion.result.txt", args.genome_id, args.grc)

    # delete intermediate files
    if debug_mode == False:
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.txt"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.filt1.txt"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.filt2.txt"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.filt3.txt"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.splicing.txt"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.splicing.contig.fa"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.splicing.contig.psl"])
        subprocess.check_call(["rm", outputFilePrefix + ".chimeric.clustered.splicing.contig.check.txt"])


def fusionfusion_main(args):

    starBamFile = args.star
    ms2BamFile = args.ms2
    th2BamFile = args.th2
    output_dir = args.out

    if starBamFile == None and ms2BamFile  == None and th2BamFile == None:
        print >> sys.stderr, "At least one of --star, --ms2 or --th2 should be included"
        sys.exit(1)
 
    # config.param_conf.read(args.param)
    # debug_mode = config.param_conf.getboolean("debug", "debug_mode")

    ##########
    # set parameters
    param_conf.reference_genome = args.reference_genome
    # param_conf.resource_dir = args.resource_dir
    param_conf.debug = args.debug
    param_conf.abnormal_insert_size = args.abnormal_insert_size
    param_conf.min_major_clipping_size = args.min_major_clipping_size
    param_conf.min_read_pair_num = args.min_read_pair_num
    param_conf.min_valid_read_pair_ratio = args.min_valid_read_pair_ratio
    param_conf.min_cover_size = args.min_cover_size
    param_conf.anchor_size_thres = args.anchor_size_thres
    param_conf.min_chimeric_size = args.min_chimeric_size
    param_conf.min_allowed_contig_match_diff = args.min_allowed_contig_match_diff
    param_conf.check_contig_size_other_breakpoint = args.check_contig_size_other_breakpoint
    param_conf.filter_same_gene = args.filter_same_gene
    ##########

    debug_mode = param_conf.debug

    ####################
    # make direcotry
    utils.make_directory(output_dir)
    ####################


    ####################
    # parsing chimeric reads from bam files
    if starBamFile is not None:

        parseJunctionInfo.parseJuncInfo_STAR(starBamFile, output_dir + "/star.chimeric.tmp.txt")
                                             

        hOUT = open(output_dir + "/star.chimeric.txt", "w")
        subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k4,4", "-k5,5n", output_dir + "/star.chimeric.tmp.txt"], stdout = hOUT)
        hOUT.close()

        cluster_filter_junction(output_dir + "/star.chimeric.txt", output_dir + "/star", args)

        if debug_mode == False:
            subprocess.check_call(["rm", output_dir + "/star.chimeric.tmp.txt"])
            subprocess.check_call(["rm", output_dir + "/star.chimeric.txt"])

    if ms2BamFile is not None:

        parseJunctionInfo.extractFusionReads_ms2(ms2BamFile, output_dir + "/ms2.chimeric.tmp.sam")

        hOUT = open(output_dir + "/ms2.chimeric.sam", "w")
        subprocess.check_call(["sort", "-k1", output_dir + "/ms2.chimeric.tmp.sam"], stdout = hOUT)
        hOUT.close()
    
        parseJunctionInfo.parseJuncInfo_ms2(output_dir + "/ms2.chimeric.sam", output_dir + "/ms2.chimeric.tmp.txt", args.abnormal_insert_size) 

        hOUT = open(output_dir + "/ms2.chimeric.txt", "w")
        subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k4,4", "-k5,5n", output_dir + "/ms2.chimeric.tmp.txt"], stdout = hOUT)
        hOUT.close()

        cluster_filter_junction(output_dir + "/ms2.chimeric.txt", output_dir + "/ms2", args)

        if debug_mode == False:
            subprocess.check_call(["rm", output_dir + "/ms2.chimeric.tmp.sam"])
            subprocess.check_call(["rm", output_dir + "/ms2.chimeric.sam"])
            subprocess.check_call(["rm", output_dir + "/ms2.chimeric.tmp.txt"])
            subprocess.check_call(["rm", output_dir + "/ms2.chimeric.txt"])


    if th2BamFile is not None:

        parseJunctionInfo.extractFusionReads_th2(th2BamFile, output_dir + "/th2.chimeric.tmp.sam")

        hOUT = open(output_dir + "/th2.chimeric.sam", "w")
        subprocess.check_call(["sort", "-k1", output_dir + "/th2.chimeric.tmp.sam"], stdout = hOUT)
        hOUT.close()
        parseJunctionInfo.parseJuncInfo_th2(output_dir + "/th2.chimeric.sam", output_dir + "/th2.chimeric.tmp.txt", args.abnormal_insert_size)
        
        hOUT = open(output_dir + "/th2.chimeric.txt", "w")
        subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k4,4", "-k5,5n", output_dir + "/th2.chimeric.tmp.txt"], stdout = hOUT)
        hOUT.close()

        cluster_filter_junction(output_dir + "/th2.chimeric.txt", output_dir + "/th2", args)

        if debug_mode == False:
            subprocess.check_call(["rm", output_dir + "/th2.chimeric.tmp.sam"])
            subprocess.check_call(["rm", output_dir + "/th2.chimeric.sam"])
            subprocess.check_call(["rm", output_dir + "/th2.chimeric.tmp.txt"])
            subprocess.check_call(["rm", output_dir + "/th2.chimeric.txt"])


    annotationFunction.merge_fusion_result(output_dir, 
                                           output_dir + "/fusion_fusion.result.txt")

    



#! /usr/bin/env python

import re
import regions, seq_utils, region_utils
import pysam

# import config
from config import *

regRe = re.compile(r'([^ \t\n\r\f\v,]+):(\d+)\-(\d+)')
ReContig = re.compile(r'([^ \t\n\r\f\v,]+):([\+\-])(\d+)\-([^ \t\n\r\f\v,]+):([\+\-])(\d+)_contig([12])')

def filterCoverRegion(inputFilePath, outputFilePath):

    # min_read_pair_num = config.param_conf.getint("filter_condition", "min_read_pair_num")
    # min_valid_read_pair_ratio = config.param_conf.getfloat("filter_condition", "min_valid_read_pair_ratio")
    # min_cover_size = config.param_conf.getint("filter_condition", "min_cover_size")
    # min_chimeric_size = config.param_conf.getint("filter_condition", "min_chimeric_size")
    # anchor_size_thres = config.param_conf.getint("filter_condition", "anchor_size_thres")
    min_read_pair_num = param_conf.min_read_pair_num
    min_valid_read_pair_ratio = param_conf.min_valid_read_pair_ratio
    min_cover_size = param_conf.min_cover_size
    min_chimeric_size = param_conf.min_chimeric_size
    anchor_size_thres = param_conf.anchor_size_thres


    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    for line in hIN:
        F = line.rstrip('\n').split('\t')

        if F[0] == F[3] and abs(int(F[1]) - int(F[4])) < min_chimeric_size: continue

        coveredRegion_primary = F[9].split(';')
        coveredRegion_pair = F[12].split(';')
        coveredRegion_SA = F[15].split(';')

        # check the number of unique read pairs
        coveredRegion_meta = [coveredRegion_primary[i] + ';' + coveredRegion_pair[i] + ';' + coveredRegion_SA[i] for i in range(0, len(coveredRegion_primary))]
        uniqueCoverdRegion_meta = list(set(coveredRegion_meta))
        if len(uniqueCoverdRegion_meta) < min_read_pair_num: continue

        # check the maximum anchor size
        coverRegionSize_primary = map(region_utils.getCoverSize, coveredRegion_primary)
        coverRegionSize_SA = map(region_utils.getCoverSize, coveredRegion_SA)
        anchor_size = [min(coverRegionSize_primary[i], coverRegionSize_SA[i]) for i in range(len(coverRegionSize_primary))]
        if max(anchor_size) < anchor_size_thres: continue

            
        # filter by the ratio of valid read pairs
        pairPos = F[17].split(';')
        if float(pairPos.count("0")) / float(len(pairPos)) > 1 - min_valid_read_pair_ratio: continue

        # check for the covered region
        pairPos = F[17].split(';')
        primaryPos = F[18].split(';')

        region1 =  regions.Regions()
        region2 =  regions.Regions()

        for i in range(0, len(coveredRegion_primary)):
            if primaryPos[i] == "1" and pairPos != "0":
                for reg in coveredRegion_primary[i].split(','):
                    region1.addMerge(reg)
            elif primaryPos[i] == "2" and pairPos != "0":
                for reg in coveredRegion_primary[i].split(','):
                    region2.addMerge(reg)

        for i in range(0, len(coveredRegion_SA)):
            if primaryPos[i] == "1" and pairPos != "0":
                for reg in coveredRegion_SA[i].split(','):
                    region2.addMerge(reg)
            elif primaryPos[i] == "2" and pairPos != "0":
                for reg in coveredRegion_SA[i].split(','):
                    region1.addMerge(reg)

        for i in range(0, len(coveredRegion_pair)):
            if (primaryPos[i] == "1" and pairPos[i] == "1") or (primaryPos[i] == "2" and pairPos[i] == "2"):
                for reg in coveredRegion_pair[i].split(','):
                    region1.addMerge(reg)
            elif (primaryPos[i] == "1" and pairPos[i] == "2") or (primaryPos[i] == "2" and pairPos[i] == "1"):
                for reg in coveredRegion_pair[i].split(','):
                    region2.addMerge(reg)

        region1.reduceMerge()
        region2.reduceMerge()

        if region1.regionSize() < min_cover_size or region2.regionSize() < min_cover_size: continue

        print >> hOUT, '\t'.join(F)

    hIN.close()
    hOUT.close()



def filterPoolControl(input_file, output_file, control_file):

    control_db = pysam.TabixFile(control_file)

    hout = open(output_file, 'w') 

    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
 
            # skip if the junction is included in the control file
            tabixErrorFlag = 0
            if control_file is not None:
                try:
                    records = control_db.fetch(F[0], int(F[1]) - 5, int(F[1]) + 5)
                except Exception as inst:
                    # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
                    # tabixErrorMsg = str(inst.args)
                    tabixErrorFlag = 1

            control_flag = 0;
            if tabixErrorFlag == 0:
                 for record_line in records:
                    record = record_line.split('\t')
                    if F[0] == record[0] and F[1] == record[1] and F[2] == record[2] and \
                       F[3] == record[3] and F[4] == record[4] and F[5] == record[5]:
                        control_flag = 1

            if control_flag == 0:
                print >> hout, '\t'.join(F)

    hout.close()


def extractSplicingPattern(inputFilePath, outputFilePath):

    # reference_genome = config.param_conf.get("alignment", "reference_genome")
    reference_genome = param_conf.reference_genome

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    for line in hIN:
        F = line.rstrip('\n').split('\t')

        # check for the covered region
        coveredRegion_primary = F[9].split(';')
        coveredRegion_pair = F[12].split(';')
        coveredRegion_SA = F[15].split(';')
        pairPos = F[17].split(';')
        primaryPos = F[18].split(';')


        ####################
        splice2count1 = {}
        splice2count2 = {}

        # get the splicing position for the primary reads 
        for i in range(0, len(coveredRegion_primary)):
            regs = coveredRegion_primary[i].split(',')
            if len(regs) == 1: continue

            chr_reg = ""
            start_reg = []
            end_reg = [] 
            for reg in regs:
                mReg = regRe.match(reg)
                chr_reg = mReg.group(1)
                start_reg.append(int(mReg.group(2)))
                end_reg.append(int(mReg.group(3)))
            
            start_reg.sort()
            end_reg.sort()
        
            for j in range(0, len(start_reg) - 1):
                regKey = (chr_reg, int(end_reg[j]), int(start_reg[j + 1]))
                if primaryPos[i] == "1" and pairPos != "0":
                    splice2count1[regKey] = (splice2count1[regKey] + 1 if regKey in splice2count1 else 1)
                elif primaryPos[i] == "2" and pairPos != "0":
                    splice2count2[regKey] = (splice2count2[regKey] + 1 if regKey in splice2count2 else 1)


        # get the splicing position for the supplementary reads
        for i in range(0, len(coveredRegion_SA)):
            regs = coveredRegion_SA[i].split(',')
            if len(regs) == 1: continue

            chr_reg = ""
            start_reg = []
            end_reg = []
            for reg in regs:
                mReg = regRe.match(reg)
                chr_reg = mReg.group(1)
                start_reg.append(int(mReg.group(2)))
                end_reg.append(int(mReg.group(3)))

            start_reg.sort()
            end_reg.sort()

            for j in range(0, len(start_reg) - 1):
                regKey = (chr_reg, int(end_reg[j]), int(start_reg[j + 1]))
                if primaryPos[i] == "1" and pairPos != "0":
                    splice2count2[regKey] = (splice2count2[regKey] + 1 if regKey in splice2count2 else 1)
                elif primaryPos[i] == "2" and pairPos != "0":
                    splice2count1[regKey] = (splice2count1[regKey] + 1 if regKey in splice2count1 else 1)


        # get the splicing position for the pair reads
        for i in range(0, len(coveredRegion_pair)):
            regs = coveredRegion_pair[i].split(',')
            if len(regs) == 1: continue

            chr_reg = ""
            start_reg = []
            end_reg = []
            for reg in regs:
                mReg = regRe.match(reg)
                chr_reg = mReg.group(1)
                start_reg.append(int(mReg.group(2)))
                end_reg.append(int(mReg.group(3)))

            start_reg.sort()
            end_reg.sort()

            for j in range(0, len(start_reg) - 1):
                regKey = (chr_reg, int(end_reg[j]), int(start_reg[j + 1]))
                if (primaryPos[i] == "1" and pairPos[i] == "1") or (primaryPos[i] == "2" and pairPos[i] == "2"):
                    splice2count1[regKey] = (splice2count1[regKey] + 1 if regKey in splice2count1 else 1)
                elif (primaryPos[i] == "1" and pairPos[i] == "2") or (primaryPos[i] == "2" and pairPos[i] == "1"):
                    splice2count2[regKey] = (splice2count2[regKey] + 1 if regKey in splice2count2 else 1)
        ####################

        ####################
        splice2count1_ref = {}
        splice2count2_ref = {}
        for key in splice2count1:
            splice2count1_ref[key] = 0

        for key in splice2count2:
            splice2count2_ref[key] = 0

        
        # check about each splicing position for the primary reads 
        for i in range(0, len(coveredRegion_primary)):
            for reg in coveredRegion_primary[i].split(','):
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
     
                if primaryPos[i] == "1" and pairPos != "0":
                    for key in splice2count1:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1

                if primaryPos[i] == "2" and pairPos != "0":
                    for key in splice2count2:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1


        # check about each splicing position for the supplementary reads 
        for i in range(0, len(coveredRegion_SA)):
            for reg in coveredRegion_SA[i].split(','):
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))

                if primaryPos[i] == "1" and pairPos != "0":
                    for key in splice2count2:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1

                if primaryPos[i] == "2" and pairPos != "0":
                    for key in splice2count1:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1


        # check about each splicing position for the pair reads 
        for i in range(0, len(coveredRegion_pair)):
            for reg in coveredRegion_pair[i].split(','):
                if reg == "*": continue
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))

                if (primaryPos[i] == "1" and pairPos[i] == "1") or (primaryPos[i] == "2" and pairPos[i] == "2"):
                    for key in splice2count1:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count1_ref[key] = splice2count1_ref[key] + 1

                if (primaryPos[i] == "1" and pairPos[i] == "2") or (primaryPos[i] == "2" and pairPos[i] == "1"):
                    for key in splice2count2:
                        chr_key, start_key, end_key  = key[0], key[1], key[2]

                        # if the given region cover arround the spliced sites
                        if F[2] == "+" and start_reg <= end_key - 5 and end_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1
                        if F[2] == "-" and start_reg <= start_key - 5 and start_key + 5 <= end_reg: splice2count2_ref[key] = splice2count2_ref[key] + 1
        ####################


        ####################
        # check whether we should adopt each splice junction
        spliceJunction1 = []
        spliceJunction2 = []
        for key in splice2count1:
            if splice2count1[key] > splice2count1_ref[key]: spliceJunction1.append(key)

        for key in splice2count2:
            if splice2count2[key] > splice2count2_ref[key]: spliceJunction2.append(key)
        ####################


        ####################
        region1 =  regions.Regions()
        region2 =  regions.Regions()

        for i in range(0, len(coveredRegion_primary)):
            if primaryPos[i] == "1" and pairPos != "0":
                for reg in coveredRegion_primary[i].split(','):
                    region1.addMerge(reg)
            elif primaryPos[i] == "2" and pairPos != "0":
                for reg in coveredRegion_primary[i].split(','):
                    region2.addMerge(reg)

        for i in range(0, len(coveredRegion_SA)):
            if primaryPos[i] == "1" and pairPos != "0":
                for reg in coveredRegion_SA[i].split(','):
                    region2.addMerge(reg)
            elif primaryPos[i] == "2" and pairPos != "0":
                for reg in coveredRegion_SA[i].split(','):
                    region1.addMerge(reg)

        for i in range(0, len(coveredRegion_pair)):
            if (primaryPos[i] == "1" and pairPos[i] == "1") or (primaryPos[i] == "2" and pairPos[i] == "2"):
                for reg in coveredRegion_pair[i].split(','):
                    region1.addMerge(reg)
            elif (primaryPos[i] == "1" and pairPos[i] == "2") or (primaryPos[i] == "2" and pairPos[i] == "1"):
                for reg in coveredRegion_pair[i].split(','):
                    region2.addMerge(reg)

        region1.reduceMerge()
        region2.reduceMerge()
        ####################


        ####################
        contig1 = []
        contig2 = []

        tempPos = int(F[1])
        if F[2] == "+":
            for key in sorted(spliceJunction1, key=lambda x: x[2], reverse=True):
                if tempPos < key[2]: continue

                minStart = float("inf")
                for reg in region1.regionVec:
                    mReg = regRe.match(reg)
                    chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                    if tempPos <= end_reg and start_reg < minStart:
                        minStart = start_reg

                if minStart != float("inf"):
                    contig1.append((chr_reg, max(key[2], minStart), tempPos))
                    if minStart > key[2]: break 
                    tempPos = key[1]

            minStart = float("inf")
            for reg in region1.regionVec:
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                if tempPos <= end_reg and start_reg < minStart:
                    minStart = start_reg

            if minStart != float("inf"):
                contig1.append((chr_reg, minStart, tempPos))

        if F[2] == "-":
            for key in sorted(spliceJunction1, key=lambda x: x[1]):
                if tempPos > key[1]: continue

                maxEnd = - float("inf")            
                for reg in region1.regionVec:
                    mReg = regRe.match(reg)
                    chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                    if start_reg <= tempPos and end_reg > maxEnd:
                        maxEnd = end_reg
       
                if maxEnd != - float("inf"):
                    contig1.append((chr_reg, tempPos, min(key[1], maxEnd)))
                    if maxEnd < key[1]: break 
                    tempPos = key[2]

            maxEnd = - float("inf")
            for reg in region1.regionVec:
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                if start_reg <= tempPos and end_reg > maxEnd:
                    maxEnd = end_reg 

            if maxEnd != - float("inf"):
                contig1.append((chr_reg, tempPos, maxEnd))


        tempPos = int(F[4])
        if F[5] == "+":
            for key in sorted(spliceJunction2, key=lambda x: x[2], reverse=True):
                if tempPos < key[2]: continue

                minStart = float("inf")
                for reg in region2.regionVec:
                    mReg = regRe.match(reg)
                    chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                    if tempPos <= end_reg and start_reg < minStart:
                        minStart = start_reg

                if minStart != float("inf"):
                    contig2.append((chr_reg, max(key[2], minStart), tempPos))
                    if minStart > key[2]: break
                    tempPos = key[1]

            minStart = float("inf")
            for reg in region2.regionVec:
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                if tempPos <= end_reg and start_reg < minStart:
                    minStart = start_reg

            if minStart != float("inf"):
                contig2.append((chr_reg, minStart, tempPos))

        if F[5] == "-":
            for key in sorted(spliceJunction2, key=lambda x: x[1]):
                if tempPos > key[1]: continue

                maxEnd = - float("inf")
                for reg in region2.regionVec:
                    mReg = regRe.match(reg)
                    chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                    if start_reg <= tempPos and end_reg > maxEnd:
                        maxEnd = end_reg
      
                if maxEnd != - float("inf"):
                    contig2.append((chr_reg, tempPos, min(key[1], maxEnd)))
                    if maxEnd < key[1]: break
                    tempPos = key[2]

            maxEnd = - float("inf")
            for reg in region2.regionVec:
                mReg = regRe.match(reg)
                chr_reg, start_reg, end_reg  = mReg.group(1), int(mReg.group(2)), int(mReg.group(3))
                if start_reg <= tempPos and end_reg > maxEnd:
                    maxEnd = end_reg

            if maxEnd != - float("inf"):
                contig2.append((chr_reg, tempPos, maxEnd))
        ####################

        contigSeq1 = ""
        contigSeq2 = ""

        inSeq = ';'.join(list(set(F[6].split(';'))))

        contigSeq1 = seq_utils.getSeq(reference_genome, contig1)
        contigSeq2 = seq_utils.getSeq(reference_genome, contig2)
        if F[2] == "+": contigSeq1 = seq_utils.reverseComplement(contigSeq1)
        if F[5] == "+": contigSeq2 = seq_utils.reverseComplement(contigSeq2)

        print >> hOUT, '\t'.join(['\t'.join(F[0:6]), str(inSeq), str(len(F[8].split(';'))), str(contig1), str(contig2), contigSeq1, contigSeq2])


    hIN.close()
    hOUT.close()



def makeJucSeqPairFa(inputFilePath, outputFilePath):

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    for line in hIN:
        F = line.rstrip('\n').split('\t')

        print >> hOUT, '>' + F[0] + ":" + F[2] + F[1] + "-" + F[3] + ":" + F[5] + F[4] + "_contig" + '1'
        print >> hOUT, F[10]

        print >> hOUT, '>' + F[0] + ":" + F[2] + F[1] + "-" + F[3] + ":" + F[5] + F[4] + "_contig" + '2'
        print >> hOUT, F[11]

    hIN.close()
    hOUT.close()



def checkMatching(inputFilePath, outputFilePath):

    # min_allowed_contig_match_diff = config.param_conf.getint("filter_condition", "min_allowed_contig_match_diff")
    # check_contig_size_other_breakpoint = config.param_conf.getint("filter_condition", "check_contig_size_other_breakpoint")
    min_allowed_contig_match_diff = param_conf.min_allowed_contig_match_diff
    check_contig_size_other_breakpoint = param_conf.check_contig_size_other_breakpoint

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    
    tempID = ""
    for line in hIN:
        F = line.rstrip('\n').split('\t')

        if re.match(r'^\d', F[0]) is None: continue

        if F[9] != tempID:
            if tempID != "":
                otherMatch = []
                for site, value in sorted(site2Match.items(), key = lambda x: x[1], reverse=True):
                    if site == targetAln: continue
                    if value >= targetScore - min_allowed_contig_match_diff: otherMatch.append(site)
                    if len(otherMatch) >= 10: break
                otherMatch_str = ("---" if len(otherMatch) == 0 else ';'.join(otherMatch))
                print >> hOUT, tempID + '\t' + targetAln + '\t' + otherMatch_str + '\t' + str(crossMatch) + '\t' + str(targetScore) + '\t' + str(targetSize)

            tempID = F[9]
            targetAln = "---"
            targetScore= 0
            site2Match = {}
            crossMatch = 0
            targetSize = "---"

        site2Match[F[13] + ':' + str(int(F[15]) + 1) + '-' + F[16]] = int(F[0])

        chr1, strand1, pos1, chr2, strand2, pos2, contigNum = "", "", "", "", "", "", 0
        contigMatch = ReContig.match(F[9])
        if contigMatch is None:
            print >> sys.stderr, "the format of the fasta name is inconsistent at"
            print >> sys.stderr, '\t'.join(F)

        contigNum = contigMatch.group(7)
        if contigNum == "1":
            chr1, strand1, pos1, chr2, strand2, pos2 = contigMatch.group(1), contigMatch.group(2), int(contigMatch.group(3)), contigMatch.group(4), contigMatch.group(5), int(contigMatch.group(6))
        elif contigNum == "2":
            chr1, strand1, pos1, chr2, strand2, pos2 = contigMatch.group(4), contigMatch.group(5), int(contigMatch.group(6)), contigMatch.group(1), contigMatch.group(2), int(contigMatch.group(3))

        if chr1 == F[13]:
            if (strand1 == "+" and F[8] == "-" and abs(pos1 - int(F[16])) < 5) or (strand1 == "-" and F[8] == "+" and abs(pos1 - 1 - int(F[15])) < 5):

                targetAln = F[13] + ':' + str(int(F[15]) + 1) + '-' + F[16] 
                targetScore= int(F[0])
                targetSize = int(F[10])

        if chr2 == F[13] and (pos2 >= int(F[15]) - check_contig_size_other_breakpoint and pos2 <= int(F[16]) + check_contig_size_other_breakpoint):
            crossMatch = float(F[0]) / float(F[10])


    hIN.close()

    # last procedure    
    if tempID != "":
        otherMatch = []
        for site, value in sorted(site2Match.items(), key = lambda x: x[1], reverse=True):
            if site == targetAln: continue
            if value >= targetScore - min_allowed_contig_match_diff: otherMatch.append(site)
            if len(otherMatch) >= 10: break
        otherMatch_str = ("---" if len(otherMatch) == 0 else ';'.join(otherMatch))
        print >> hOUT, tempID + '\t' + targetAln + '\t' + otherMatch_str + '\t' + str(crossMatch) + '\t' + str(targetScore) + '\t' + str(targetSize)

    hOUT.close()



def filterContigCheck(inputFilePath, outputFilePath, checkMatchFile):

    # min_cover_size = config.param_conf.getint("filter_condition", "min_cover_size")
    min_cover_size = param_conf.min_cover_size

    hIN = open(checkMatchFile, 'r')
    key2match1 = {}
    key2match2 = {}
    for line in hIN:
        F = line.rstrip('\n').split('\t')
        keyMatch = ReContig.match(F[0])

        chr1, strand1, pos1, chr2, strand2, pos2, contigNum = keyMatch.group(1), keyMatch.group(2), keyMatch.group(3), keyMatch.group(4), keyMatch.group(5), keyMatch.group(6), keyMatch.group(7)

        if contigNum == "1":
            key2match1['\t'.join([chr1, pos1, strand1, chr2, pos2, strand2])] = '\t'.join(F[1:6])

        if contigNum == "2":
            key2match2['\t'.join([chr1, pos1, strand1, chr2, pos2, strand2])] = '\t'.join(F[1:6])

    hIN.close()


    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    for line in hIN:
        F = line.rstrip('\n').split('\t')

        key = '\t'.join(F[0:6])

        match1 = (key2match1[key] if key in key2match1 else "---\t---\t---\t---\t---")
        match2 = (key2match2[key] if key in key2match2 else "---\t---\t---\t---\t---") 

        matches1 = match1.split('\t')
        matches2 = match2.split('\t')

        if matches1[0] == "---" or matches2[0] == "---": continue
        if matches1[1] != "---" or matches2[1] != "---": continue
        if float(matches1[2]) > 0 or float(matches2[2]) > 0: continue
        # if int(matches1[3]) < min_cover_size or int(matches2[3]) < min_cover_size: continue

        print >> hOUT, key + '\t' + F[6] + '\t' + F[7] + '\t' +  match1 + '\t' + match2

    hIN.close()
    hOUT.close()




#! /usr/bin/env python

import sys
import re
import cigar_utils
import pysam
import collections
# import config
from config import *

# for mapsplice2
ReFus_ms2 = re.compile('FUS_(\d+)_(\d+)\(([\-\+])([\-\+])\)')

# for TopHat2
ReFus_th2 = re.compile('XF:Z:(\d) ([^ \t\n\r\f\v,]+)\-([^ \t\n\r\f\v,]+) (\d+) ([\dMmNnIiDd]+[Mm])(\d+F)([\dMmNnIiDd]+[Mm])')

# for STAR
cigarSRe_right = re.compile('(\d+)S$')
cigarSRe_left = re.compile('^(\d+)S')


def extractFusionReads_ms2(inputFilePath, outputFilePath):

    inBamFile = pysam.AlignmentFile(inputFilePath, "r") 

    fusionReadID = {}
    for read in inBamFile.fetch():

        for item in read.tags:
            if item[0] == "ZF" and ReFus_ms2.search(item[1]) is not None:
                fusionReadID[read.qname] = 1

    
    # it seem that reset can be used only for indexed bam files...
    # inBamFile.reset() # 

    inBamFile.close()

    inBamFile = pysam.AlignmentFile(inputFilePath, "r")
    outBamFile = pysam.AlignmentFile(outputFilePath, "w", template = inBamFile)
    for read in inBamFile.fetch():
        
        if read.qname in fusionReadID:
            outBamFile.write(read)


    inBamFile.close()
    outBamFile.close()

    
def extractFusionReads_th2(inputFilePath, outputFilePath):

    inBamFile = pysam.AlignmentFile(inputFilePath, "r")

    fusionReadID = {}
    for read in inBamFile.fetch():

        # get the flag information
        flags = format(int(read.flag), "#014b")[:1:-1]

        # skip if either of the read pair is unmapped
        if flags[2] == "1" or flags[3] == "1": continue

        # skip supplementary alignment
        if flags[8] == "1" or flags[11] == "1": continue

        # skip duplicated reads
        if flags[10] == "1": continue

        for item in read.tags:
            if item[0] == "XF":
                fusionReadID[read.qname] = 1


    inBamFile.close()

    inBamFile = pysam.AlignmentFile(inputFilePath, "r")
    outBamFile = pysam.AlignmentFile(outputFilePath, "w", template = inBamFile)
    for read in inBamFile.fetch():

        # get the flag information
        flags = format(int(read.flag), "#014b")[:1:-1]

        # skip supplementary alignment
        if flags[8] == "1" or flags[11] == "1": continue


        if read.qname in fusionReadID:
            outBamFile.write(read)


    inBamFile.close()
    outBamFile.close()



def getFusInfo_ms2(tempID, tempLine, fusInfo):

    # abnormal_insert_size = config.param_conf.getint("parse_condition", "abnormal_insert_size")
    abnormal_insert_size = param_conf.abnormal_insert_size

    # check the fusion validity
    ufusInfo = list(set(fusInfo))
    for fus in ufusInfo:
        if fus == 0: continue

        chr_primary, pos_primary, dir_primary, chr_pair, pos_pair, dir_pair, chr_chimera, pos_chimera, dir_chimera = "*", "*", "*", "*", "*", "*", "*", "*", "*"
        mq_primary, cover_primary, mq_pair, cover_pair, mq_chimera, cover_chimera = "*", "*", "*", "*", "*", "*"
        # index showing the order of fusion 
        # 1: FUS_(breakPos_primary)_(breakPos_chimera)_(breakDir_primary)_(breakDir_chimera),
        # -1: FUS_(breakPos_chimera)_(breakPos_primary)_(breakDir_chimera)_(breakDir_primary)
        fusOrder = 0
        
        for i in range(0, len(tempLine)):
             
            FF = tempLine[i].split('\t')
            flags = format(int(FF[1]), "#014b")[:1:-1]
            if flags[8] == "1" and fusInfo[i] == fus:
                chr_chimera = FF[2]
                pos_chimera = FF[3]
                dir_chimera = ("+" if flags[4] != "1" else "-")
                mq_chimera = FF[4]
                cover_chimera = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])
                if fusOrder == 0: fusOrder = -1
            elif flags[8] != "1" and fusInfo[i] == fus:
                chr_primary = FF[2]
                pos_primary = FF[3]
                dir_primary = ("+" if flags[4] != "1" else "-")
                mq_primary = FF[4]
                cover_primary = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])
                if fusOrder == 0: fusOrder = 1
            elif flags[8] != "1" and flags[2] != "1" and (fusInfo[i] != fus or chr_primary != "*"):
                chr_pair = FF[2]
                pos_pair = FF[3]
                dir_pair = ("+" if flags[4] != "1" else "-")
                mq_pair = FF[4]
                cover_pair = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])

        if chr_primary == "*": continue
        fusSplit = fus.split(',')
        if fusOrder == 1:
            breakPos_primary, breakPos_chimera, breakDir_primary, breakDir_chimera = fusSplit[0], fusSplit[1], fusSplit[2], fusSplit[3]
            breakDir_chimera = ("+" if breakDir_chimera == "-" else "-") 
        else:
            breakPos_primary, breakPos_chimera, breakDir_primary, breakDir_chimera = fusSplit[1], fusSplit[0], fusSplit[3], fusSplit[2]
            breakDir_primary = ("+" if breakDir_primary == "-" else "-")

        pairPos = 0
        if pos_pair != "*":
            if breakDir_primary == "+" and dir_pair == "+" and int(breakPos_primary) - abnormal_insert_size <= int(pos_pair) <= int(breakPos_primary): pairPos = 1
            if breakDir_primary == "-" and dir_pair == "-" and int(breakPos_primary) <= int(pos_pair) <= int(breakPos_primary) + abnormal_insert_size: pairPos = 1
            if breakDir_chimera == "+" and dir_pair == "+" and int(breakPos_chimera) - abnormal_insert_size <= int(pos_pair) <= int(breakPos_chimera): pairPos = 2
            if breakDir_chimera == "-" and dir_pair == "-" and int(breakPos_chimera) <= int(pos_pair) <= int(breakPos_chimera) + abnormal_insert_size: pairPos = 2

        if chr_primary < chr_chimera or chr_primary == chr_chimera and breakPos_primary <= breakPos_chimera:
            return '\t'.join([chr_primary, breakPos_primary, breakDir_primary, chr_chimera, breakPos_chimera, breakDir_chimera, "---", tempID, \
                             mq_primary, cover_primary, dir_primary, mq_pair, cover_pair, dir_pair, \
                             mq_chimera, cover_chimera, dir_chimera, str(pairPos), "1"] )
        else:
            return '\t'.join([chr_chimera, breakPos_chimera, breakDir_chimera, chr_primary, breakPos_primary, breakDir_primary, "---", tempID, \
                             mq_primary, cover_primary, dir_primary, mq_pair, cover_pair, dir_pair, \
                             mq_chimera, cover_chimera, dir_chimera, str(pairPos), "2"] )



def getFusInfo_th2(tempID, tempLine, fusInfo, SAFlag):

    # abnormal_insert_size = config.param_conf.getint("parse_condition", "abnormal_insert_size")
    abnormal_insert_size = param_conf.abnormal_insert_size

    # check the fusion validity
    ufusInfo = list(set(fusInfo))
    for fus in ufusInfo:
        if fus == '*': continue

        chr_primary, pos_primary, dir_primary, chr_pair, pos_pair, dir_pair, chr_chimera, pos_chimera, dir_chimera = "*", "*", "*", "*", "*", "*", "*", "*", "*"
        mq_primary, cover_primary, mq_pair, cover_pair, mq_chimera, cover_chimera = "*", "*", "*", "*", "*", "*"

        for i in range(0, len(tempLine)):

            FF = tempLine[i].split('\t')
            if FF[0] == "HWI-ST1165:73:AD0JHGACXX:7:1102:18080:166569":
                pass

            flags = format(int(FF[1]), "#014b")[:1:-1]
            if fusInfo[i] == fus:
                if str(SAFlag[i]) == "1":
                    if chr_primary == '*':
                        chr_primary = FF[2]
                        pos_primary = FF[3]
                        dir_primary = ("+" if flags[4] != "1" else "-")
                        mq_primary = FF[4]
                        cover_primary = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])
                    else:
                        chr_pair = FF[2]
                        pos_pair = FF[3]
                        dir_pair = ("+" if flags[4] != "1" else "-")
                        mq_pair = FF[4]
                        cover_pair = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])
                else:
                    chr_chimera = FF[2]
                    pos_chimera = FF[3]
                    dir_chimera = ("+" if flags[4] != "1" else "-")
                    mq_chimera = FF[4]
                    cover_chimera = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])
            else:
                chr_pair = FF[2]
                pos_pair = FF[3]
                dir_pair = ("+" if flags[4] != "1" else "-")
                mq_pair = FF[4]
                cover_pair = cigar_utils.getCoverRegion(FF[2], FF[3], FF[5])


        if chr_primary == "*": continue
        fusSplit = fus.split(',')
        cigar_primary = fusSplit[3]
        cigar_chimera = fusSplit[5] 
        
        breakDir_primary = ("-" if cigar_primary.islower() else "+")
        breakDir_chimera = ("+" if cigar_chimera.islower() else "-")
        breakPos_primary = str(pos_primary if cigar_primary.islower() else cigar_utils.getEndPos(pos_primary, cigar_primary.upper()))
        breakPos_chimera = str(cigar_utils.getEndPos(pos_chimera, cigar_chimera.upper()) if cigar_chimera.islower() else pos_chimera)

        pairPos = 0
        if pos_pair != "*":
            if chr_primary == chr_pair and breakDir_primary == "+" and dir_pair == "+" and int(breakPos_primary) - abnormal_insert_size <= int(pos_pair) <= int(breakPos_primary): pairPos = 1
            if chr_primary == chr_pair and breakDir_primary == "-" and dir_pair == "-" and int(breakPos_primary) <= int(pos_pair) <= int(breakPos_primary) + abnormal_insert_size: pairPos = 1
            if chr_chimera == chr_pair and breakDir_chimera == "+" and dir_pair == "+" and int(breakPos_chimera) - abnormal_insert_size <= int(pos_pair) <= int(breakPos_chimera): pairPos = 2
            if chr_chimera == chr_pair and breakDir_chimera == "-" and dir_pair == "-" and int(breakPos_chimera) <= int(pos_pair) <= int(breakPos_chimera) + abnormal_insert_size: pairPos = 2

        if pairPos == 0: continue

        if chr_primary < chr_chimera or chr_primary == chr_chimera and breakPos_primary <= breakPos_chimera:
            return '\t'.join([chr_primary, breakPos_primary, breakDir_primary, chr_chimera, breakPos_chimera, breakDir_chimera, "---", tempID, \
                             mq_primary, cover_primary, dir_primary, mq_pair, cover_pair, dir_pair, \
                             mq_chimera, cover_chimera, dir_chimera, str(pairPos), "1"] )
        else:
            return '\t'.join([chr_chimera, breakPos_chimera, breakDir_chimera, chr_primary, breakPos_primary, breakDir_primary, "---", tempID, \
                             mq_primary, cover_primary, dir_primary, mq_pair, cover_pair, dir_pair, \
                             mq_chimera, cover_chimera, dir_chimera, str(pairPos), "2"] )


def parseJuncInfo_ms2(inputFilePath, outputFilePath):

    """
    script for collecting short reads supporting fusion candidates in MapSplice2 sam file
    """

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    tempID = ""
    fusFlag = [] 
    fusInfo = []
    tempLine = []

    for line in hIN:
        if line[0] == "@": continue
        line = line.rstrip('\n')
        F = line.split('\t')

        if tempID != F[0]:
            if fusInfo.count(0) != len(tempLine):
                tempFusInfo = getFusInfo_ms2(tempID, tempLine, fusInfo)
                if tempFusInfo is not None:
                    print >> hOUT, tempFusInfo

            tempID = F[0]
            fusFlag = []
            fusInfo = [] 
            tempLine = [] 
    
        tempLine.append(line)
        mFus = ReFus_ms2.search('\t'.join(F[11:]))
        if mFus is not None:
            fusFlag.append(1)
            fusInfo.append(','.join([mFus.group(1), mFus.group(2), mFus.group(3), mFus.group(4)]))
        else:
            fusFlag.append(0)
            fusInfo.append(0)

    hIN.close()


    if fusInfo.count(0) != len(tempLine):
        tempFusInfo = getFusInfo_ms2(tempID, tempLine, fusInfo)
        if tempFusInfo is not None:
            print >> hOUT, tempFusInfo

    hOUT.close()


def parseJuncInfo_th2(inputFilePath, outputFilePath):

    """
    script for collecting short reads supporting fusion candidates in TopHat2 sam file
    """

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    tempID = ""
    fusInfo = []
    SAFlag = []
    tempLine = []

    for line in hIN:
        if line[0] == "@": continue
        line = line.rstrip('\n')
        F = line.split('\t')

        if tempID != F[0]:
            if fusInfo.count('*') != len(tempLine):
                tempFusInfo = getFusInfo_th2(tempID, tempLine, fusInfo, SAFlag)
                if tempFusInfo is not None:
                    print >> hOUT, tempFusInfo

            tempID = F[0]
            fusInfo = []
            SAFlag = []
            tempLine = []

        tempLine.append(line)
        mFus = ReFus_th2.search('\t'.join(F[11:]))
        if mFus is not None:
            fusInfo.append(','.join([mFus.group(2), mFus.group(3), mFus.group(4), mFus.group(5), mFus.group(6), mFus.group(7)]))
            SAFlag.append(mFus.group(1))
        else:
            fusInfo.append('*')
            SAFlag.append('*')

    hIN.close()


    if fusInfo.count('*') != len(tempLine):
        tempFusInfo = getFusInfo_th2(tempID, tempLine, fusInfo, SAFlag)
        if tempFusInfo is not None:
            print >> hOUT, tempFusInfo


    hOUT.close()


def getFusInfo_STAR(juncLine):

    # abnormal_insert_size = config.param_conf.getint("parse_condition", "abnormal_insert_size")
    # min_major_clip_size = config.param_conf.getint("parse_condition", "min_major_clip_size")
    abnormal_insert_size = param_conf.abnormal_insert_size
    min_major_clip_size = param_conf.min_major_clipping_size
 
    """
    function for organizing and print junction information
    """

    # be reminded necessary variables
    chr_primary, pos_primary, dir_primary, chr_pair, pos_pair, dir_pair, chr_SA, pos_SA, dir_SA = "", "", "", "", "", "", "", "", ""
    mq_primary, coverRegion_primary, mq_pair, coverRegion_pair, mq_SA, coverRegion_SA = "", "", "", "", "", ""
    juncChr_primary, juncPos_primary, juncDir_primary, juncChr_SA, juncPos_SA, juncDir_SA = "", "", "", "", "", "" 

    # collect about the supplementary alingment information
    right_clipping_SA = 0
    left_clipping_SA = 0
    flags_SA = ""
    for line in juncLine:
        F = line.split('\t')
        flags = format(int(F[1]), "#014b")[:1:-1] 
        if flags[8] != "1": continue

        if chr_SA != "":
            print >> sys.stderr, "Multiple supplementary alignment at:" + '\n' +  '\n'.join(juncLine)

        chr_SA = F[2]
        juncChr_SA = F[2]
        pos_SA = int(F[3])
        dir_SA = ("-" if flags[4] == "1" else "+")
        mq_SA = F[4]
        coverRegion_SA = cigar_utils.getCoverRegion(F[2], F[3], F[5])
        endPos_SA = cigar_utils.getEndPos(pos_SA, F[5])

        flags_SA = flags
        if flags_SA[6] == flags_SA[7]: print >> sys.stderr, "The supplementary Read is both first and second reads at:" + '\n' + '\n'.join(juncLine)

        tempMatch = cigarSRe_right.search(F[5])
        if tempMatch is not None: right_clipping_SA = int(tempMatch.group(1))

        tempMatch = cigarSRe_left.search(F[5])
        if tempMatch is not None: left_clipping_SA = int(tempMatch.group(1))

    # collect information about primary junction read and its pair read
    right_clipping_primary = 0
    left_clipping_primary = 0
    cigar_primary = ""
    readLength_primary = 0
    readID_primary = ""
    for line in juncLine:
        F = line.split('\t')
        flags = format(int(F[1]), "#014b")[:1:-1]
        if flags[8] == "1": continue

        # primary junction read
        if flags[6] == flags_SA[6] and flags[7] == flags_SA[7]:
            chr_primary = F[2]
            juncChr_primary = F[2]
            pos_primary = int(F[3])
            dir_primary = ("-" if flags[4] == "1" else "+")
            mq_primary = F[4]
            coverRegion_primary = cigar_utils.getCoverRegion(F[2], F[3], F[5])
            readLength_primary = len(F[9])
            endPos_primary = cigar_utils.getEndPos(pos_primary, F[5])
            readID_primary = F[0] + ("/1" if flags[6] == "1" else "/2")

            tempMatch = cigarSRe_right.search(F[5])
            if tempMatch is not None: right_clipping_primary = int(tempMatch.group(1))
        
            tempMatch = cigarSRe_left.search(F[5])
            if tempMatch is not None: left_clipping_primary = int(tempMatch.group(1))

        elif flags[6] == flags_SA[7] and flags[7] == flags_SA[6]:
            chr_pair = F[2]
            juncChr_pair = F[2]
            pos_pair = int(F[3])
            dir_pair = ("-" if flags[4] == "1" else "+")
            mq_pair = F[4]
            coverRegion_pair = cigar_utils.getCoverRegion(F[2], F[3], F[5])
        else:
            print >> sys.stderr, "The following read is both first and second reads at:" + '\n' + line


    if right_clipping_primary >= min_major_clip_size:

        clipLen_primary = right_clipping_primary

        juncChr_primary = chr_primary
        juncPos_primary = endPos_primary
        juncDir_primary = "+"

        expected_clipLen_SA = readLength_primary - clipLen_primary
        expected_clipDir_SA = ("-" if dir_primary== dir_SA else "+")

        validFlag = 0
        juncDir_SA = ""
        juncPos_SA = ""

        # the pair read is aligned at the same chromosome with the primary read
        if dir_primary == "-" and dir_pair == "+" and chr_primary == chr_pair and 0 <= pos_primary - pos_pair < abnormal_insert_size:

            if dir_SA == "+" and expected_clipDir_SA == "+" and right_clipping_SA > 0:
                clipLen_SA = right_clipping_SA
                juncDir_SA = "+"
                juncPos_SA = endPos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA - (expected_clipLen_SA - clipLen_SA)
                juncType = 1
                validFlag = 1

            if dir_SA == "-" and expected_clipDir_SA == "-" and left_clipping_SA > 0:
                clipLen_SA = left_clipping_SA
                juncDir_SA = "-"
                juncPos_SA = int(pos_SA)
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA + (expected_clipLen_SA - clipLen_SA)
                juncType = 1
                validFlag = 1

        # when the supplementary read is aligned on the same chromosome with the paired read
        if dir_primary == "+" and chr_SA == chr_pair:

            if dir_SA == "-" and dir_pair == "+" and 0 <= pos_SA - pos_pair < abnormal_insert_size and expected_clipDir_SA == "+" and right_clipping_SA > 0:
                clipLen_SA = right_clipping_SA
                juncDir_SA = "+"
                juncPos_SA = endPos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA - (expected_clipLen_SA - clipLen_SA)
                juncType = 2
                validFlag = 1

            if dir_SA == "+" and dir_pair == "-" and 0 <= pos_pair - pos_SA < abnormal_insert_size and expected_clipDir_SA == "-" and left_clipping_SA > 0:
                clipLen_SA = left_clipping_SA
                juncDir_SA = "-"
                juncPos_SA = pos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA + (expected_clipLen_SA - clipLen_SA)
                juncType = 2
                validFlag = 1

        if validFlag == 1:

            juncSurplus = "---"
            if clipLen_SA > expected_clipLen_SA:
                surPlus_start = readLength_primary - clipLen_primary
                surPlus_end = surPlus_start + clipLen_SA - expected_clipLen_SA
                juncSurplus = read.seq[surPlus_start:surPlus_end]

            # reorder by the chromosome position and print
            if juncChr_primary < juncChr_SA or juncChr_primary == juncChr_SA and juncPos_primary <= juncPos_SA:
                return '\t'.join([juncChr_primary, str(juncPos_primary), juncDir_primary, juncChr_SA, str(juncPos_SA), juncDir_SA, \
                                 juncSurplus, readID_primary, mq_primary, coverRegion_primary, dir_primary, \
                                 mq_pair, coverRegion_pair, dir_pair, mq_SA, coverRegion_SA, dir_SA, str(juncType) , "1"])

            else:
                return '\t'.join([juncChr_SA, str(juncPos_SA), juncDir_SA, juncChr_primary, str(juncPos_primary), juncDir_primary, \
                                 juncSurplus, readID_primary, mq_primary, coverRegion_primary, dir_primary, \
                                 mq_pair, coverRegion_pair, dir_pair, mq_SA, coverRegion_SA, dir_SA, str(juncType) , "2"])


    if left_clipping_primary >= min_major_clip_size:

        clipLen_primary = left_clipping_primary
        juncChr_primary = chr_primary
        juncPos_primary = int(pos_primary)
        juncDir_primary = "-"
        juncChr_SA = chr_SA

        expected_clipLen_SA = readLength_primary - clipLen_primary
        expected_clipDir_SA = ("+" if dir_primary== dir_SA else "-")

        validFlag = 0
        juncDir_SA = ""
        juncPos_SA = ""
        # the pair read is aligned at the same chromosome with the primary read
        if dir_primary == "+" and dir_pair == "-" and chr_primary == chr_pair and 0 <= pos_pair - pos_primary < abnormal_insert_size:

            if dir_SA == "+" and expected_clipDir_SA == "+" and right_clipping_SA > 0:
                clipLen_SA = right_clipping_SA
                juncDir_SA = "+"
                juncPos_SA = endPos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA - (expected_clipLen_SA - clipLen_SA)
                juncType = 1
                validFlag = 1

            if dir_SA == "-" and expected_clipDir_SA == "-" and left_clipping_SA > 0:
                clipLen_SA = left_clipping_SA
                juncDir_SA = "-"
                juncPos_SA = pos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA + (expected_clipLen_SA - clipLen_SA)
                juncType = 1
                validFlag = 1

        if dir_primary == "-" and chr_SA == chr_pair:

            if dir_SA == "-" and dir_pair == "+" and 0 <= pos_SA - pos_pair < abnormal_insert_size and expected_clipDir_SA == "+" and right_clipping_SA > 0:
                clipLen_SA = right_clipping_SA
                juncDir_SA = "+"
                juncPos_SA = endPos_SA 
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA - (expected_clipLen_SA - clipLen_SA)
                juncType = 2
                validFlag = 1

            if dir_SA == "+" and dir_pair == "-" and 0 <= pos_pair - pos_SA < abnormal_insert_size and expected_clipDir_SA == "-" and left_clipping_SA:
                clipLen_SA = left_clipping_SA
                juncDir_SA = "-"
                juncPos_SA = pos_SA
                if clipLen_SA < expected_clipLen_SA: juncPos_SA = juncPos_SA + (expected_clipLen_SA - clipLen_SA)
                juncType = 2
                validFlag = 1


        if validFlag == 1:

            juncSurplus = "---"
            if clipLen_SA > expected_clipLen_SA:
                surPlus_end = clipLen_primary # this is right
                surPlus_start = surPlus_end - (clipLen_SA - expected_clipLen_SA)
                juncSurplus = read.seq[surPlus_start:surPlus_end]

            # reorder by the chromosome position and print
            if juncChr_primary < juncChr_SA or juncChr_primary == juncChr_SA and juncPos_primary <= juncPos_SA:
                return '\t'.join([juncChr_primary, str(juncPos_primary), juncDir_primary, juncChr_SA, str(juncPos_SA), juncDir_SA, \
                                 juncSurplus, readID_primary, mq_primary, coverRegion_primary, dir_primary, \
                                 mq_pair, coverRegion_pair, dir_pair, mq_SA, coverRegion_SA, dir_SA, str(juncType) , "1"])
                                 
            else:                
                return '\t'.join([juncChr_SA, str(juncPos_SA), juncDir_SA, juncChr_primary, str(juncPos_primary), juncDir_primary, \
                                 juncSurplus, readID_primary, mq_primary, coverRegion_primary, dir_primary, \
                                 mq_pair, coverRegion_pair, dir_pair, mq_SA, coverRegion_SA, dir_SA, str(juncType) , "2"])



def parseJuncInfo_STAR(inputFilePath, outputFilePath):

     
    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    tempID = ""
    tempLine = []
    for line in hIN:
        line = line.rstrip('\n')
        F = line.split('\t')
        if F[0][0] == "@": continue

        if tempID != F[0]:
            if tempID != "" and len(tempLine) == 3:
                tempFusInfo = getFusInfo_STAR(tempLine)
                if tempFusInfo is not None:
                    print >> hOUT, tempFusInfo

            tempID = F[0]
            tempLine = []

        tempLine.append(line)

    hIN.close()


    if tempID != "" and len(tempLine) == 3:
        tempFusInfo = getFusInfo_STAR(tempLine)
        if tempFusInfo is not None:
            print >> hOUT, tempFusInfo


    hOUT.close()



def clusterJuncInfo(inputFilePath, outputFilePath):

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    # I believe this is appropriate..
    check_margin_size = 30

    cluster_junc = {}
    cluster_inseq, cluster_id, cluster_pair_pos, cluster_primary_pos = {}, {}, {}, {}
    cluster_MQ_primary, cluser_cover_primary, cluster_dir_primary = {}, {}, {}
    cluster_MQ_pair, cluster_cover_pair, cluster_dir_pair = {}, {}, {}
    cluster_MQ_SA, cluster_cover_SA, cluster_dir_SA = {}, {}, {}

    for line in hIN:
 
        F = line.rstrip('\n').split('\t')
        match = 0
        delList = []

        for key in sorted(cluster_junc):
            tchr1, tpos1, tdir1, tchr2, tpos2, tdir2, inseq_size = key.split('\t')

            # the investigated key is sufficiently far from the current line in the input file and no additional line to merge is expected. therefore flush the key and information
            if F[0] != tchr1 or int(F[1]) > int(tpos1) + check_margin_size:

                # obtain the most frequent junction
                junc_counter = collections.Counter(cluster_junc[key])
                best_junc = junc_counter.most_common(1)[0][0]
                btchr1, btpos1, btdir1, btchr2, btpos2, btdir2, btinseq = best_junc.split('\t') 

                print >> hOUT, '\t'.join([btchr1, btpos1, btdir1, btchr2, btpos2, btdir2, btinseq, \
                                    ';'.join(cluster_id[key]), ';'.join(cluster_MQ_primary[key]), ';'.join(cluser_cover_primary[key]), ';'.join(cluster_dir_primary[key]), \
                                    ';'.join(cluster_MQ_pair[key]), ';'.join(cluster_cover_pair[key]), ';'.join(cluster_dir_pair[key]), \
                                    ';'.join(cluster_MQ_SA[key]), ';'.join(cluster_cover_SA[key]), ';'.join(cluster_dir_SA[key]), \
                                    ';'.join(cluster_pair_pos[key]), ';'.join(cluster_primary_pos[key])])


                # add to the deletion list (later the key will be removed from the dictionaries)
                delList.append(key)
                continue

            else:

                # check whether the investigated key and the current line should be merged or not
                if F[0] == tchr1 and F[3] == tchr2 and F[2] == tdir1 and F[5] == tdir2:

                    flag = 0
                    temp_seq_size = 0 if F[6] == "---" else len(F[6])
                    # detailed check on the junction position considering inserted sequences
                    if F[2] == "+":
                        expected_diff_size = (int(F[1]) - int(tpos1)) + (temp_seq_size - int(inseq_size))
                        if (F[5] == "+" and int(F[4]) == int(tpos2) - expected_diff_size) or (F[5] == "-" and int(F[4]) == int(tpos2) + expected_diff_size):
                            flag = 1
                    else:
                        expected_diff_size = (int(F[1]) - int(tpos1)) + (int(inseq_size) - temp_seq_size)
                        if (F[5] == "+" and int(F[4]) == int(tpos2) + expected_diff_size) or (F[5] == "-" and int(F[4]) == int(tpos2) - expected_diff_size):
                            flag = 1

                    if flag == 1:

                        match = 1
                        cluster_inseq[key].append(F[6])
                        cluster_id[key].append(F[7])
                        cluster_MQ_primary[key].append(F[8])
                        cluser_cover_primary[key].append(F[9])
                        cluster_dir_primary[key].append(F[10])
                        cluster_MQ_pair[key].append(F[11])
                        cluster_cover_pair[key].append(F[12])
                        cluster_dir_pair[key].append(F[13])
                        cluster_MQ_SA[key].append(F[14])
                        cluster_cover_SA[key].append(F[15])
                        cluster_dir_SA[key].append(F[16])
                        cluster_pair_pos[key].append(F[17])
                        cluster_primary_pos[key].append(F[18])

                        # whether to check whether the inserted sequence should be reverse-complemented?
                        cluster_junc[key].append('\t'.join(F[0:7]))

        for key in delList:
            del cluster_junc[key]
            del cluster_inseq[key]
            del cluster_id[key]
            del cluster_MQ_primary[key]
            del cluser_cover_primary[key]
            del cluster_dir_primary[key]
            del cluster_MQ_pair[key]
            del cluster_cover_pair[key]
            del cluster_dir_pair[key]
            del cluster_MQ_SA[key]
            del cluster_cover_SA[key]
            del cluster_dir_SA[key]
            del cluster_pair_pos[key]
            del cluster_primary_pos[key]

        # if the current line in the input file does not match any of the pooled keys
        if match == 0:
            temp_seq_size = 0 if F[6] == "---" else len(F[6])
            key = '\t'.join(F[0:6] + [str(temp_seq_size)])
            # whether to check whether the inserted sequence should be reverse-complemented?
            cluster_junc[key] = ['\t'.join(F[0:7])]

            cluster_inseq[key], cluster_id[key] = [F[6]], [F[7]]
            cluster_MQ_primary[key], cluser_cover_primary[key], cluster_dir_primary[key] = [F[8]], [F[9]], [F[10]]
            cluster_MQ_pair[key], cluster_cover_pair[key], cluster_dir_pair[key] = [F[11]], [F[12]], [F[13]]
            cluster_MQ_SA[key], cluster_cover_SA[key], cluster_dir_SA[key] = [F[14]], [F[15]], [F[16]]
            cluster_pair_pos[key], cluster_primary_pos[key] = [F[17]], [F[18]]

    hIN.close()

    for key in sorted(cluster_junc):

        # obtain the most frequent junction
        junc_counter = collections.Counter(cluster_junc[key])
        best_junc = junc_counter.most_common(1)[0][0]
        btchr1, btpos1, btdir1, btchr2, btpos2, btdir2, btinseq = best_junc.split('\t')

        print >> hOUT, '\t'.join([btchr1, btpos1, btdir1, btchr2, btpos2, btdir2, btinseq, \
                            ';'.join(cluster_id[key]), ';'.join(cluster_MQ_primary[key]), ';'.join(cluser_cover_primary[key]), ';'.join(cluster_dir_primary[key]), \
                            ';'.join(cluster_MQ_pair[key]), ';'.join(cluster_cover_pair[key]), ';'.join(cluster_dir_pair[key]), \
                            ';'.join(cluster_MQ_SA[key]), ';'.join(cluster_cover_SA[key]), ';'.join(cluster_dir_SA[key]), \
                            ';'.join(cluster_pair_pos[key]), ';'.join(cluster_primary_pos[key])])


    hOUT.close()





#!/usr/bin/env python

import re

regRe = re.compile(r'([^ \t\n\r\f\v,]+):(\d+)\-(\d+)')

def getCoverSize(cover_str):
    """
    get the size of covered bases from cover strings (e.g., chr1:741208-741271,chr1:745447-745482)
    """
    size = 0
    F = cover_str.split(",")
    for i in range(len(F)):
        mReg = regRe.match(F[i])
        chr = mReg.group(1)
        start = int(mReg.group(2))
        end = int(mReg.group(3))
        size = size + end - start + 1

    return size




#! /usr/bin/env python

import sys, pysam, os, subprocess
import annot_utils.gene
import annot_utils.exon

# import config
from config import *

junction_margin = 5


def get_gene_info(chr, pos, ref_gene_tb, ens_gene_tb):

    # check gene annotation for refGene 
    tabixErrorFlag = 0
    try:
        records = ref_gene_tb.fetch(chr, int(pos) - 1, int(pos) + 1)
    except Exception as inst:
        # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
        tabixErrorFlag = 1
        
    gene = [];
    if tabixErrorFlag == 0:
        for record_line in records:
            record = record_line.split('\t')
            gene.append(record[3])

    # if any gene cannot be found in refGene, then search ensGene
    if len(gene) == 0:
        tabixErrorFlag = 0
        try:
            records = ens_gene_tb.fetch(chr, int(pos) - 1, int(pos) + 1)
        except Exception as inst:
            # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
            tabixErrorFlag = 1
           
        """ 
        # for ensGene, just the longest gene is shown
        temp_length = 0
        temp_gene = ""
        if tabixErrorFlag == 0:
            for record_line in records:
                record = record_line.split('\t')
                if int(record[4]) > temp_length:
                     temp_gene = record[3]
    
            if temp_gene != "": gene.append(temp_gene)
        """

        if tabixErrorFlag == 0:
            for record_line in records:
                record = record_line.split('\t')
                gene.append(record[3])

 
    if len(gene) == 0: gene.append("---")

    return list(set(gene))


def get_junc_info(chr, pos, ref_exon_tb, ens_exon_tb, junction_margin):

    # check exon annotation for refGene 
    tabixErrorFlag = 0
    try:
        records = ref_exon_tb.fetch(chr, int(pos) - junction_margin, int(pos) + junction_margin)
    except Exception as inst:
        # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
        tabixErrorFlag = 1
        
    junction = []
    if tabixErrorFlag == 0:
        for record_line in records:
            record = record_line.split('\t')
            if abs(int(pos) - int(record[1])) < junction_margin:
                if record[5] == "+": junction.append(record[3] + ".start")
                if record[5] == "-": junction.append(record[3] + ".end")
            if abs(int(pos) - int(record[2])) < junction_margin:
                if record[5] == "+": junction.append(record[3] + ".end")
                if record[5] == "-": junction.append(record[3] + ".start")

    # if any exon-intron junction cannot be found in refGene, then search ensGene
    if len(junction) == 0: 
        tabixErrorFlag = 0
        try:
            records = ens_exon_tb.fetch(chr, int(pos) - junction_margin, int(pos) + junction_margin)
        except Exception as inst:
            # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
            tabixErrorFlag = 1

        """             
        # for ensGene, just the longest gene is shown
        temp_length = 0
        temp_junc = ""
        if tabixErrorFlag == 0:
            for record_line in records:
                record = record_line.split('\t')
                # if int(record[4]) > temp_length:
                if True:
                    if abs(int(pos) - int(record[1])) < junction_margin:
                        if record[5] == "+": temp_junc = record[3] + ".start"
                        if record[5] == "-": temp_junc = record[3] + ".end"
                    if abs(int(pos) - int(record[2])) < junction_margin: 
                        if record[5] == "+": temp_junc = record[3] + ".end"
                        if record[5] == "-": temp_junc = record[3] + ".start"
                
            if temp_junc != "": junction.append(temp_junc)
        """

        if tabixErrorFlag == 0:
            for record_line in records:
                record = record_line.split('\t')
                if abs(int(pos) - int(record[1])) < junction_margin:
                    if record[5] == "+": junction.append(record[3] + ".start")
                    if record[5] == "-": junction.append(record[3] + ".end")
                if abs(int(pos) - int(record[2])) < junction_margin: 
                    if record[5] == "+": junction.append(record[3] + ".end")
                    if record[5] == "-": junction.append(record[3] + ".start")
    
                
    if len(junction) == 0: junction.append("---")
    
    return list(set(junction))



def filterAndAnnotation(inputFilePath, outputFilePath, genome_id, is_grc):

    hIN = open(inputFilePath, 'r')
    hOUT = open(outputFilePath, 'w')

    # annotation_dir = config.param_conf.get("annotation", "annotation_dir")
    # filter_same_gene = config.param_conf.getboolean("filter_condition", "filter_same_gene")
    # annotation_dir = param_conf.resource_dir
    filter_same_gene = param_conf.filter_same_gene

    """
    # old procedure
    # ref_gene_bed = annotation_dir + "/refGene.bed.gz"
    ref_exon_bed = annotation_dir + "/refExon.bed.gz"
    ens_gene_bed = annotation_dir + "/ensGene.bed.gz"
    ens_exon_bed = annotation_dir + "/ensExon.bed.gz"
    grch2ucsc_file = annotation_dir + "/grch2ucsc.txt"

    # relationship between CRCh and UCSC chromosome names
    grch2ucsc = {}
    with open(grch2ucsc_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            grch2ucsc[F[0]] = F[1]

    ref_gene_tb = pysam.TabixFile(ref_gene_bed)
    ref_exon_tb = pysam.TabixFile(ref_exon_bed)
    ens_gene_tb = pysam.TabixFile(ens_gene_bed)
    ens_exon_tb = pysam.TabixFile(ens_exon_bed)
    """

    annot_utils.gene.make_gene_info(outputFilePath + ".tmp.refGene.bed.gz", "refseq", genome_id, is_grc, False) 
    annot_utils.gene.make_gene_info(outputFilePath + ".tmp.ensGene.bed.gz", "gencode", genome_id, is_grc, False)
    annot_utils.exon.make_exon_info(outputFilePath + ".tmp.refExon.bed.gz", "refseq", genome_id, is_grc, False)
    annot_utils.exon.make_exon_info(outputFilePath + ".tmp.ensExon.bed.gz", "gencode", genome_id, is_grc, False)
 
    ref_gene_tb = pysam.TabixFile(outputFilePath + ".tmp.refGene.bed.gz")
    ens_gene_tb = pysam.TabixFile(outputFilePath + ".tmp.ensGene.bed.gz")
    ref_exon_tb = pysam.TabixFile(outputFilePath + ".tmp.refExon.bed.gz")
    ens_exon_tb = pysam.TabixFile(outputFilePath + ".tmp.ensExon.bed.gz")

    for line in hIN:

        F = line.rstrip('\n').split('\t')
    
        # check gene annotation for the side 1
        gene1 = get_gene_info(F[0], F[1], ref_gene_tb, ens_gene_tb)
        
        # check gene annotation for the side 2
        gene2 = get_gene_info(F[3], F[4], ref_gene_tb, ens_gene_tb)

        # check exon-intron junction annotation for the side 1 
        junction1 = get_junc_info(F[0], F[1], ref_exon_tb, ens_exon_tb, junction_margin) 

        # check exon-intron junction annotation for the side 2 
        junction2 = get_junc_info(F[3], F[4], ref_exon_tb, ens_exon_tb, junction_margin)

        sameGeneFlag = 0
        for g1 in gene1:
            for g2 in gene2:
                if g1 == g2 and g1 != "---": sameGeneFlag = 1

        if filter_same_gene == True and sameGeneFlag == 1: continue

        print >> hOUT, '\t'.join(F[0:8]) + '\t' + ';'.join(gene1) + '\t' + ';'.join(junction1) + '\t' + ';'.join(gene2) + '\t' + ';'.join(junction2) + '\t' + \
                         F[11] + '\t' + F[12] + '\t' + F[16] + '\t' + F[17]

    hIN.close()
    hOUT.close()


    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.refGene.bed.gz"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.ensGene.bed.gz"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.refExon.bed.gz"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.ensExon.bed.gz"])

    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.refGene.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.ensGene.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.refExon.bed.gz.tbi"])
    subprocess.check_call(["rm", "-rf", outputFilePath + ".tmp.ensExon.bed.gz.tbi"])



def merge_fusion_result(input_dir, output_file_path):

    fus2count_ms2 = {}
    fus2count_star = {}
    fus2count_th2 = {}

    label_ms2, label_star, label_th2 = 0, 0, 0

    if os.path.exists(input_dir + "/ms2.fusion.result.txt"):
        label_ms2 = 1
        with open(input_dir + "/ms2.fusion.result.txt") as hIN:
            for line in hIN:
                F = line.rstrip('\n').split('\t')
                key = '\t'.join(F[0:7] + F[8:12])
                fus2count_ms2[key] = F[7]

    if os.path.exists(input_dir + "/star.fusion.result.txt"): 
        label_star = 1
        with open(input_dir + "/star.fusion.result.txt") as hIN:
            for line in hIN:
                F = line.rstrip('\n').split('\t')
                key = '\t'.join(F[0:7] + F[8:12])
                fus2count_star[key] = F[7]

    if os.path.exists(input_dir + "/th2.fusion.result.txt"):
        label_th2 = 1 
        with open(input_dir + "/th2.fusion.result.txt") as hIN:
            for line in hIN:
                F = line.rstrip('\n').split('\t')
                key = '\t'.join(F[0:7] + F[8:12])
                fus2count_th2[key] = F[7]

    fus_keys = list(set(fus2count_star.keys() + fus2count_ms2.keys() + fus2count_th2.keys()))
    hOUT = open(output_file_path, 'w')

    for fus in fus_keys:
        count_star = fus2count_star[fus] if fus in fus2count_star else "---"
        count_ms2 = fus2count_ms2[fus] if fus in fus2count_ms2 else "---"
        count_th2 = fus2count_th2[fus] if fus in fus2count_th2 else "---"

        print_line = fus
        if label_ms2 == 1: print_line = print_line + '\t' + count_ms2
        if label_star == 1: print_line = print_line + '\t' + count_star
        if label_th2 == 1: print_line = print_line + '\t' + count_th2

        print >> hOUT, print_line

    hOUT.close()



#! /usr/bin/env python

import re

regionRe = re.compile('([^ \t\n\r\f\v,]+):(\d+)\-(\d+)')

class Regions(object):

    regionVec = []
    def __init__(self):
        self.regionVec = []

    def addMerge(self, newRegion):
        mergedFlag = 0    
        for i in range(0, len(self.regionVec)):
            tempReg = self.regionVec[i]
            newReg = regionMerge(self.regionVec[i], newRegion)
            if newReg != '':
                mergedFlag = 1
                self.regionVec[i] = newReg

        if mergedFlag == 0:
            self.regionVec.append(newRegion)    


    def reduceMerge(self):
        mergedFlag = 1
        while mergedFlag == 1:
            mergedFlag = 0
            mergePair = []
            for i in range(0, len(self.regionVec) - 1):
                for j in range(i + 1, len(self.regionVec)):
                    newReg = regionMerge(self.regionVec[i], self.regionVec[j])
                    if newReg != '':
                        mergedFlag = 1 
                        mergePair = [i, j]
                        
            if mergedFlag == 1:
                self.regionVec[mergePair[0]] = regionMerge(self.regionVec[mergePair[0]], self.regionVec[mergePair[1]])
                del self.regionVec[mergePair[1]]



    def regionSize(self):
        size = 0
        for reg in self.regionVec:
            reMatch = regionRe.match(reg)
            chr = reMatch.group(1)
            start = int(reMatch.group(2))
            end = int(reMatch.group(3))
            size += end - start + 1
        return(size)


def regionMerge(region1, region2):

    reMatch1 = regionRe.match(region1)
    reMatch2 = regionRe.match(region2)
    chr1 = reMatch1.group(1)
    start1 = int(reMatch1.group(2))
    end1 = int(reMatch1.group(3))
    chr2 = reMatch2.group(1)
    start2 = int(reMatch2.group(2))
    end2 = int(reMatch2.group(3))
    
    # if there is no overlap, then return region for the 
    if chr1 != chr2 or end1 < start2 or end2 < start1:
        return '' 
        
    return chr1 + ":" + str(min(start1, start2)) + "-" + str(max(end1, end2))
####################
##########



#! /usr/bin/env python

import re

reCigar = re.compile(r'(\d+)([DIMNS])')

# get the region covered by the alignment
def getCoverRegion(chr, pos, cigar):

    tempStart = int(pos)
    tempSize = 0 
    coverRegion = []
    for m in reCigar.finditer(cigar):
        if m.group(2) == "N":
            coverRegion.append(chr + ":" + str(tempStart) + "-" + str(tempStart + tempSize - 1))
            tempStart = tempStart + tempSize + int(m.group(1))
            tempSize = 0 
        elif m.group(2) in ["M", "D"]:
            tempSize = int(tempSize) + int(m.group(1))

    coverRegion.append(chr + ":" + str(tempStart) + "-" + str(tempStart + tempSize - 1))

    return ','.join(coverRegion)


# get the end positon
def getEndPos(pos, cigar):

    tempPos = int(pos) - 1
    for m in reCigar.finditer(cigar):
        if m.group(2) in ["M", "N", "D"]:
            tempPos = tempPos + int(m.group(1))

    return tempPos 


